from typing import Optional


class TablePicture(object):
    def __init__(
        self,
        store_id,           # type: str
        table_info=None,    # type: Optional[dict]
    ):
        self.store_code = store_id
        self.table_info = table_info
