from typing import Optional


class UnlockVoucherRequest(object):
    def __init__(
        self,               # type: UnlockVoucherRequest
        benefit_id,         # type: str
        order_picture,      # type: str
        table_picture,      # type: Optional[str]
    ):
        # type: (...) -> None

        self.benefit_id = benefit_id
        self.order_picture = order_picture
        self.table_picture = table_picture
