from helper import ExtendedEnum


class GetAndLockCouponErrorCode(ExtendedEnum):
    COUPON_NOT_FOUND = "The voucher id was not found on the partner"
    PARTNER_CUSTOM_ERROR = "The voucher is not available based on custom partner rules"
    COUPON_LOCKED = "The voucher is being used in another pos"
    COUPON_ALREADY_USED = "The voucher was already used"
