from typing import List

from dailygoals_services.model import SaleChallenge
from dailygoals_services.model.repository import DailyGoalsRepository


class GetSalesChallengeInteractor:
    def __init__(self, daily_goals_repository):
        # type: (DailyGoalsRepository) -> None

        self.daily_goals_repository = daily_goals_repository

    def get_sales_challenge(self, limit):
        # type: (int) -> List[SaleChallenge]

        return self.daily_goals_repository.get_sales_challenge(limit)
