from ._TagLinesCommand import TagLinesCommand


class AddTagLinesCommand(TagLinesCommand):
    def __init__(self, order_id, view, line_ids, tag, command_order):
        super(AddTagLinesCommand, self).__init__(order_id, view, line_ids, tag, command_order)

    def get_hash_extra_value(self):
        # type: () -> str

        return "AddTagLinesCommand{}{}".format(self.line_ids, self.tag)
