from production.box import ProductionBox
from typing import Union, List


class ProductionTree(object):
    def __init__(self, name, root, leaves):
        # type: (str, ProductionBox, Union[ProductionBox, List[ProductionBox]]) -> None
        self.name = name
        self.root = root
        self.leaves = leaves
        self.tree_name = ''

    def set_tree_name(self, tree_name):
        # type: (str) -> None

        self.tree_name = tree_name
