from ._MessageBusExceptionType import MessageBusExceptionType
from ._InnerException import InnerException


class MessageBusException(InnerException):
    def __init__(self, type, ex=None):
        # type: (MessageBusExceptionType, Exception) -> None
        super(MessageBusException, self).__init__(ex)
        self.type = type
