from helper import ExtendedEnum

from ._LoyaltyToken import LoyaltyToken


class ListenedToken(ExtendedEnum):
    TK_LOYALTY_GET_LOYALTY_ID = LoyaltyToken.create_token(number="1")
    TK_LOYALTY_GET_AND_LOCK_VOUCHER = LoyaltyToken.create_token(number="2")
    TK_LOYALTY_BURN_VOUCHER = LoyaltyToken.create_token(number="3")
    TK_LOYALTY_UNLOCK_VOUCHER = LoyaltyToken.create_token(number="4")
    TK_LOYALTY_GET_ID_TYPE = LoyaltyToken.create_token(number="5")
    TK_LOYALTY_LOCK_BENEFIT = LoyaltyToken.create_token(number="6")
    TK_LOYALTY_GET_ENDPOINTS_CONFIG = LoyaltyToken.create_token(number="7")
    TK_LOYALTY_GET_PAYMENT_BENEFIT = LoyaltyToken.create_token(number="8")
    TK_LOYALTY_GET_TOTALED_MESSAGE = LoyaltyToken.create_token(number="9")

    TK_LOYALTY_IS_AVAILABLE = LoyaltyToken.create_token(number="FF")
