from typing import Optional


class PaymentPicture(object):
    def __init__(
        self,
        store_id,             # type: str
        payment_info=None,    # type: Optional[dict]
    ):
        # type: (...) -> None

        self.store_code = store_id
        self.payment_info = payment_info
