class PaymentInfo(object):
    def __init__(
        self,
        tender_id,          # type: int
        tender_amount,      # type: float
        tender_desc,        # type: str
        tender_type,        # type: int
        tender_detail,      # type: str
        change_amount,      # type: float
    ):
        self.tender_id = tender_id
        self.tender_amount = tender_amount
        self.tender_desc = tender_desc
        self.tender_type = tender_type
        self.tender_detail = tender_detail
        self.change_amount = change_amount


class GetBenefitOnPaymentRequest(object):
    def __init__(
        self,                   # type: GetBenefitOnPaymentRequest
        pos_id,                 # type: int
        loyalty_customer_id,    # type: str
        payment_info,           # type: PaymentInfo
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.loyalty_customer_id = loyalty_customer_id
        self.payment_info = payment_info
