from typing import Optional


class FeeDetails(object):

    def __init__(
        self,                   # type: FeeDetails
        code,                   # type: str
        product_code=None,      # type: Optional[int]
        product_name=None,      # type: Optional[str]
    ):
        # type: (...) -> None

        self.code = code
        self.product_code = product_code
        self.product_name = product_name
